pub struct AssetKeySetSentinelKey { /* private fields */ }Expand description
Sentinel for tracking all asset keys of a type.
Used by list_asset_keys to track dependencies on the set of asset keys,
rather than individual asset values.
Implementations